﻿using UnityEngine;
using UnityEngine.AzureSky;
using HarmonyLib;


public class TimeOfDay : Mod
{
    private static AzureSkyController SkyController;
    private const int _defaultDayCycle = 20;
    private const int _stopTimeDayCycle = 0;
    private static bool IsTimeStopped = false;
    private static float PreviousSetHour;

    public void Start()
    {
        SkyController = FindObjectOfType<AzureSkyController>();
        Debug.Log("Mod TimeOfDay has been loaded!");
    }

    public override void WorldEvent_WorldLoaded()
    {
        SkyController = FindObjectOfType<AzureSkyController>();

        if (IsTimeStopped)
        {
            StopTime();
            SetTime(new string[] { PreviousSetHour.ToString() });
        }
    }

    public void OnModUnload()
    {
        StartTime();
        Debug.Log("Mod TimeOfDay has been unloaded!");
    }

    [ConsoleCommand(name: "getHour", docs: "Gets the current hour.")]
    public static string GetHour()
    {
        return SkyController.timeOfDay.hour.ToString();
    }

    [ConsoleCommand(name: "setTime", docs: "Set the current time, between 1-24")]
    public static string SetTime(string[] args)
    {
        var value = float.Parse(args[0]);

        if (value > 24 || value < 1)
            return "Failed to set time. Value must be between 1-24";

        FindObjectOfType<AzureSkyController>().timeOfDay.GotoTime(value);

        PreviousSetHour = value;

        return $"Time set to {value}";
    }

    [ConsoleCommand(name: "stopTime", docs: "Stops time from progressing")]
    public static string StopTime()
    {
        SkyController.timeOfDay.dayCycle = _stopTimeDayCycle;
        Traverse.Create(SkyController).Field<float>("m_timeProgression").Value = SkyController.timeOfDay.GetDayLength();
        IsTimeStopped = true;

        return $"Time has been stopped";
    }

    [ConsoleCommand(name: "startTime", docs: "Starts time progression")]
    public static string StartTime()
    {
        SkyController.timeOfDay.dayCycle = _defaultDayCycle;
        Traverse.Create(SkyController).Field<float>("m_timeProgression").Value = SkyController.timeOfDay.GetDayLength();
        IsTimeStopped = false;

        return $"Time has been started";
    }
}